//+------------------------------------------------------------------+
//|                               Test_SYMBOL_TRADE_FREEZE_LEVEL.mq5 |
//|                        Copyright 2016, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2016, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.00"
#property description "      SYMBOL_TRADE_FREEZE_LEVEL"
//---     
#include <Trade\Trade.mqh> 
CTrade trade;
//---     
#define Ask    SymbolInfoDouble(_Symbol,SYMBOL_ASK)
#define Bid    SymbolInfoDouble(_Symbol,SYMBOL_BID)
#define  OrderType (ENUM_ORDER_TYPE)OrderGetInteger(ORDER_TYPE)
//---  SYMBOL_TRADE_STOPS_LEVEL  SYMBOL_TRADE_FREEZE_LEVEL
int freeze_level,stops_level;
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//---    ,        
   freeze_level=(int)SymbolInfoInteger(_Symbol,SYMBOL_TRADE_FREEZE_LEVEL);
   if(freeze_level!=0)
     {
      PrintFormat("SYMBOL_TRADE_FREEZE_LEVEL=%d:     ,"+
                  "   %d    ",freeze_level,freeze_level);
     }
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//|                              |
//+------------------------------------------------------------------+
double GetActivationPrice(ulong ticket)
  {
//---
   double activation_price=0;
//---
   if(OrderSelect(ticket))
     {
      ENUM_ORDER_TYPE type=OrderType;
      //---    Ask
      if(type==ORDER_TYPE_BUY_LIMIT || type==ORDER_TYPE_BUY_STOP)
        {
         activation_price=Ask;
        }
      else
      //---    Bid
      if(type==ORDER_TYPE_SELL_LIMIT || type==ORDER_TYPE_SELL_STOP)
        {
         activation_price=Bid;
        }
     }
//---      
   return activation_price;
  }
//+------------------------------------------------------------------+
//|                     |
//+------------------------------------------------------------------+
double GetNearestPrice(ENUM_ORDER_TYPE type,int distance)
  {
   double price=0;
//---   
   switch(type)
     {
      case  ORDER_TYPE_SELL_STOP:
         price=Bid-distance*_Point;
         break;
      case  ORDER_TYPE_BUY_LIMIT:
         price=Ask-distance*_Point;
         break;
      case  ORDER_TYPE_SELL_LIMIT:
         price=Bid+distance*_Point;
         break;
      case  ORDER_TYPE_BUY_STOP:
         price=Ask+distance*_Point;
         break;
      default:
         break;
     }
//---   
   return(NormalizeDouble(price,_Digits));
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
   int orders=OrdersTotal();
//---       
   int distance,ticket;
//---  
   double activation_price;
//---         
   if(orders==1)
     {
      //---    
      if((ticket=(int)OrderGetTicket(0))!=0)
        {
         activation_price=GetActivationPrice(ticket);
         //---         
         distance=(int) MathRound(MathAbs(activation_price-OrderGetDouble(ORDER_PRICE_OPEN))/_Point);
         //---         
         freeze_level=(int)SymbolInfoInteger(_Symbol,SYMBOL_TRADE_FREEZE_LEVEL);
         //---          
         stops_level=(int)SymbolInfoInteger(_Symbol,SYMBOL_TRADE_STOPS_LEVEL);
         //---   -      
         int min=MathMax(freeze_level,stops_level);
         double nearest_price=GetNearestPrice(OrderType,min+1);
         //---    
         double openprice=OrderGetDouble(ORDER_PRICE_OPEN);
         //---   StopLoss  TakeProfit
         double sl=OrderGetDouble(ORDER_SL);
         double tp=OrderGetDouble(ORDER_TP);
         datetime expiration=(datetime)OrderGetInteger(ORDER_TIME_EXPIRATION);
         ENUM_ORDER_TYPE_TIME time_type=(ENUM_ORDER_TYPE_TIME)OrderGetInteger(ORDER_TYPE_TIME);
         //---     
         if(distance<freeze_level)
           {
            PlaySound("stops.wav"); //   
            Comment(StringFormat("Distance=%d [SYMBOL_TRADE_FREEZE_LEVEL=%d]  %s  CurrentPrice=%.5f",
                    distance,freeze_level,EnumToString(OrderType),activation_price));
            //---        ,   
            if(MathAbs(nearest_price-openprice)>_Point)
              {
               //---     
               PrintFormat("Try to modify %s #%d at %.5f ==> new price=%.5f:  Bid=%.5f  Ask=%.5f",
                           EnumToString(OrderType),ticket,openprice,nearest_price,Bid,Ask);

               //---    
               if(!trade.OrderModify(ticket,nearest_price,sl,tp,time_type,0))
                 {
                  PrintFormat("freeze_level=%d",freeze_level);
                  //---   
                  trade.ResultRetcodeDescription();
                 }
               else
                  PrintFormat("Modification of order %s #%d done successfully",EnumToString(OrderType),ticket);
               Print(" -------------");
              }
            //---       -        
            return;
           }
         if(distance>2*min)
           {
            //---         
            nearest_price=GetNearestPrice(OrderType,stops_level+1);
            //---        
            int current_delta=(int)MathAbs((openprice-activation_price)/_Point);
            //---       
            if(MathAbs(nearest_price-openprice)/_Point>=1)
              {

               //---   
               PrintFormat("Modify order %s #%d at %.5f => %.5f  OpenPrice-CurrenPrice=%.5f-%.5f=%d points Ask=%.5f Bid=%.5f",
                           EnumToString(OrderType),ticket,openprice,nearest_price,openprice,activation_price,current_delta,Ask,Bid);
               //--- 
               if(trade.OrderModify(ticket,nearest_price,sl,tp,time_type,expiration))
                 {
                  PrintFormat("Order %s #%d modified succesfully",EnumToString(OrderType),ticket);
                 }
               //---   
               else
                 {
                  trade.PrintResult();
                 }
              }
            //---        -        
            return;
           }
         //---     
        }
     }
//--- OnTick() 
  }
//+------------------------------------------------------------------+
